﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO.IsolatedStorage;
using System.Linq;
using System.Net;
using System.Net.NetworkInformation;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Effects;
using System.Windows.Shapes;
using System.Windows.Threading;
using System.Xml.Linq;

namespace SilverTwitMSDN
{
    public partial class Page : UserControl
    {
        #region Fields

        private readonly string domain = "localhost:6124"; //TODO:  Domain for the SilverTwitMSDNWS
        private readonly string userName = "USERNAME"; // TODO: Twitter UserName should be entered into the SilverTwit app through a configuration screen.
        private readonly string password = "PASSWORD";  // TODO: Twitter Password should be entered into the SilverTwit app through a configuration screen.
        private TweetType tweetType = TweetType.All;

        private const string TWEET_STORAGE = "SilverTwitTweetStorage";
        private OOBStats stats = new OOBStats();
        private DispatcherTimer timer;

        #endregion

        #region Enums

        private enum TweetType
        {
            Replies,
            All,
            DirectMessages
        }

        #endregion

        #region Properties

        public string BaseUri
        {
            get { return string.Format("http://{0}/TwitterRESTService.svc", domain);  }
        }

        #endregion

        #region Constructors

        public Page()
        {
            InitializeComponent();
            this.Loaded += Page_Loaded;
            PostTweetButton.Click += PostTweetButton_Click;
            DirectMessageButton.Click += DirectMessageButton_Click;
            ReplyButton.Click += ReplyButton_Click;
            ShowRepliesButton.Click += ShowRepliesButton_Click;
            ShowDirectMessagesButton.Click += ShowDirectMessagesButton_Click;
            ShowTimelineButton.Click += ShowTimelineButton_Click;
            TweetTextBox.KeyDown += TweetTextBox_KeyDown;
            TweetTextBox.TextChanged += TweetTextBox_TextChanged;

            InstallButton.Click += new RoutedEventHandler(InstallButton_Click);
            NetworkChange.NetworkAddressChanged += new NetworkAddressChangedEventHandler(NetworkChange_NetworkAddressChanged);
            Application.Current.InstallStateChanged += new EventHandler(Network_InstallStateChanged);

            SetOutOfBrowserStats();
            StartTimer();
            this.DataContext = stats;
        }


        #endregion

        #region Out of Browser Events

        private void NetworkChange_NetworkAddressChanged(object sender, EventArgs e)
        {
            stats.IsNetworkAvailable = NetworkInterface.GetIsNetworkAvailable();

            if (NetworkInterface.GetIsNetworkAvailable())
            {
                // get the stored tweet, try to post it
                if (IsolatedStorageSettings.ApplicationSettings.Contains(TWEET_STORAGE))
                {
                    PostTweet(IsolatedStorageSettings.ApplicationSettings[TWEET_STORAGE].ToString());
                }
            }
        }

        private void InstallButton_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                Application.Current.Install(); // take the app OOB
            }
            catch (InvalidOperationException opex)
            {
                MessageBox.Show("Application is already Installed to the desktop.");
            }
            catch (Exception ex)
            {
                MessageBox.Show("Application could not be Installed to the desktop." + Environment.NewLine + ex.Message);
            }
        }

        private void Network_InstallStateChanged(object sender, EventArgs e)
        {
            SetNetworkInstallState();
        }

        #endregion 

        #region Out of Browser Helpers

        private void SetOutOfBrowserStats()
        {
            SetNetworkInstallState();
            stats.IsNetworkAvailable = NetworkInterface.GetIsNetworkAvailable();
            SetOfflineStatus();
        }

        private void SetOfflineStatus()
        {
            if (Application.Current.IsRunningOutOfBrowser)
                stats.RunningModeMessage = "Running Out of Browser";
            else
                stats.RunningModeMessage = "Running In Browser";
        }

        private bool CheckNetworkStatus()
        {
            if (NetworkInterface.GetIsNetworkAvailable())
                return true;

            MessageBox.Show("A network connection cannot be detected.");
            return false;
        }

        private void SetNetworkInstallState()
        {
            InstallState installState = Application.Current.InstallState;
            switch (installState)
            {
                case InstallState.NotInstalled:
                    stats.InstallStateMessage = "Running Online";
                    break;
                case InstallState.Installing:
                    stats.InstallStateMessage = "Installing";
                    break;
                case InstallState.Installed:
                    stats.InstallStateMessage = "Installed";
                    break;
                case InstallState.InstallFailed:
                    stats.InstallStateMessage = "Install Failed";
                    break;
                default:
                    throw new ArgumentOutOfRangeException();
            }
        }

        #endregion

        #region Event Handlers
        
        private void timer_Tick(object sender, EventArgs e)
        {
            RefreshTweets();
        }

        private void Page_Loaded(object sender, RoutedEventArgs e)
        {
            tweetType = TweetType.All;
            RefreshTweets();
        }

        private void PostTweetButton_Click(object sender, RoutedEventArgs e)
        {
            //if (!CheckNetworkStatus()) return;

            PostTweet();
        }

        private void ShowRepliesButton_Click(object sender, RoutedEventArgs e)
        {
            if (!CheckNetworkStatus()) return;

            tweetType = TweetType.Replies;
            RefreshTweets();
        }

        private void ShowDirectMessagesButton_Click(object sender, RoutedEventArgs e)
        {
            if (!CheckNetworkStatus()) return;

            tweetType = TweetType.DirectMessages;
            RefreshTweets();
        }

        private void ShowTimelineButton_Click(object sender, RoutedEventArgs e)
        {
            if (!CheckNetworkStatus()) return;

            tweetType = TweetType.All;
            RefreshTweets();
        }

        private void ReplyButton_Click(object sender, RoutedEventArgs e)
        {
            InsertTinyButtonItem("@");
        }

        private void DirectMessageButton_Click(object sender, RoutedEventArgs e)
        {
            InsertTinyButtonItem("d");
        }

        private void TweetTextBox_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.Enter)
            {
                PostTweet();
                PostTweetButton.Focus();
            }
        }

        private void TweetTextBox_TextChanged(object sender, TextChangedEventArgs e)
        {
            if (TweetTextBox.Text.Length > 140)
                TweetTextBox.Text = TweetTextBox.Text.Substring(0, 140);
        }

        #endregion

        #region Helper Methods

        private void StartTimer()
        {
            timer = new DispatcherTimer();
            timer.Interval = new TimeSpan(0, 3, 0);
            timer.Tick += timer_Tick;
            timer.Start();
        }

        private void InsertTinyButtonItem(string symbol)
        {
            TweetStatus tweet = TweetsListBox.SelectedItem as TweetStatus;
            if (tweet == null)
                return;

            TweetTextBox.Focus();

            if (TweetTextBox.Text.Length == 0)
                TweetTextBox.Text = string.Format("{0}{1}", symbol, tweet.User.ScreenName);
            else
                TweetTextBox.Text += string.Format(" {0}{1}", symbol, tweet.User.ScreenName);
        }

        private void LoadDirectMessages(XElement xmlElement)
        {
            XNamespace ns = "";
            var twitterQuery = from msg in xmlElement.Descendants(ns + "direct_message")
               let sender = msg.Element(ns + "sender")
               select new TweetStatus
               {
                   TwitterId = (msg.Element(ns + "id").Value).ToLong(),
                   CreatedAt = (msg.Element(ns + "created_at").Value).ToDateTime(),
                   Text = msg.Element(ns + "text").Value,
                   //Source = "",
                   //Truncated = "",
                   //InReplyToStatusId = "",
                   //InReplyToUserId = "",
                   //Favorited = "",
                   User = new User
                   {
                       UserId = (sender.Element(ns + "id").Value).ToLong(),
                       Name = sender.Element(ns + "name").Value,
                       ScreenName = sender.Element(ns + "screen_name").Value,
                       Description = sender.Element(ns + "description").Value,
                       Location = sender.Element(ns + "location").Value,
                       ProfileImageUrl = sender.Element(ns + "profile_image_url").Value,
                       Url = sender.Element(ns + "url").Value,
                       Protected = (sender.Element(ns + "protected").Value).ToBool(),
                       FollowersCount = (sender.Element(ns + "followers_count").Value).ToLong()
                   }
               };
            List<TweetStatus> statusList = twitterQuery.ToList<TweetStatus>();
            TweetsListBox.DataContext = statusList;
        }

        private void LoadTweets(XElement xmlElement)
        {
            XNamespace ns = "";
            var twitterQuery = from status in xmlElement.Descendants(ns + "status")
                               let user = status.Element(ns + "user")
                               select new TweetStatus
                               {
                                   TwitterId = (status.Element(ns + "id").Value).ToLong(),
                                   CreatedAt = (status.Element(ns + "created_at").Value).ToDateTime(),
                                   Text = status.Element(ns + "text").Value,
                                   Source = status.Element(ns + "source").Value,
                                   Truncated = Convert.ToBoolean(status.Element(ns + "truncated").Value),
                                   InReplyToStatusId = status.Element(ns + "in_reply_to_status_id").Value,
                                   InReplyToUserId = status.Element(ns + "in_reply_to_user_id").Value,
                                   Favorited = (status.Element(ns + "favorited").Value).ToBool(),
                                   User = new User
                                   {
                                       UserId = (user.Element(ns + "id").Value).ToLong(),
                                       Name = user.Element(ns + "name").Value,
                                       ScreenName = user.Element(ns + "screen_name").Value,
                                       Description = user.Element(ns + "description").Value,
                                       Location = user.Element(ns + "location").Value,
                                       ProfileImageUrl = user.Element(ns + "profile_image_url").Value,
                                       Url = user.Element(ns + "url").Value,
                                       Protected = (user.Element(ns + "protected").Value).ToBool(),
                                       FollowersCount = (user.Element(ns + "followers_count").Value).ToLong()
                                   }
                               };
            List<TweetStatus> statusList = twitterQuery.ToList<TweetStatus>();
            TweetsListBox.DataContext = statusList;
        }

        #endregion

        #region Asynchronous Invocations

        private void RefreshTweets()
        {
            TweetsListBox.Opacity = .5;
            TweetsListBox.Effect = new BlurEffect { Radius = 3 };

            string tweetTypeText = string.Empty;
            switch (tweetType)
            {
                case TweetType.All:
                    tweetTypeText = "all";
                    break;
                case TweetType.DirectMessages:
                    tweetTypeText = "dm";
                    break;
                case TweetType.Replies:
                    tweetTypeText = "replies";
                    break;
            }

            string uriString = string.Format("{0}/Tweet/{1}/{2}/?pw={3}", BaseUri, userName, tweetTypeText, password);
            Uri uri = new Uri(uriString);

            WebClient svc = new WebClient();
            svc.DownloadStringCompleted += SvcDownloadStringCompleted;
            svc.DownloadStringAsync(uri);
        }

        private void PostTweet(string tweetText)
        {
            string uriString = string.Format("{0}/Tweet/{1}/?pw={2}", BaseUri, userName, password);
            Uri uri = new Uri(uriString);
            WebClient svc = new WebClient();
            svc.Headers["Content-type"] = "application/xml";
            svc.Encoding = Encoding.UTF8;
            string xml = string.Format("<TweetStatus><Text>{0}</Text></TweetStatus>", tweetText);
            svc.UploadStringCompleted += SvcUploadStringCompleted;
            svc.UploadStringAsync(uri, "POST", xml);
        }

        private void PostTweet()
        {
            string tweetText = TweetTextBox.Text;
            if (!NetworkInterface.GetIsNetworkAvailable())
            {
                if (IsolatedStorageSettings.ApplicationSettings.Contains(TWEET_STORAGE))
                {
                    MessageBox.Show(
                        "Network connection is not available and only 1 tweet can be stored for delayed posting to Twitter.");
                }
                else
                {
                    IsolatedStorageSettings.ApplicationSettings.Add(TWEET_STORAGE, tweetText);
                    MessageBox.Show(
                        "Network connection is not available. The post will be stored until connectivity returns.");
                }
                return;
            }

            PostTweet(tweetText);
        }

        #endregion

        #region Asynchronous Completion Handlers

        private void SvcDownloadStringCompleted(object sender, DownloadStringCompletedEventArgs e)
        {
            try
            {
                XElement xmlElement = XElement.Parse(e.Result);
                if (xmlElement.Descendants("direct_message").Any())
                    LoadDirectMessages(xmlElement);
                else
                    LoadTweets(xmlElement);
            }
            catch(Exception ex)
            {
                // TODO: log the exception
                //MessageBox.Show(ex.Message);
                Console.WriteLine(ex.Message);
            }
            finally
            {
                TweetsListBox.Effect = null;
                TweetsListBox.Opacity = 1;
            }
        }

        private void SvcUploadStringCompleted(object sender, UploadStringCompletedEventArgs e)
        {
            if (e.Error == null)
            {
                RefreshTweets();
                if (IsolatedStorageSettings.ApplicationSettings.Contains(TWEET_STORAGE))
                    IsolatedStorageSettings.ApplicationSettings.Remove(TWEET_STORAGE);
                TweetTextBox.Text = "";
            }
            else
            {
                //TODO: Have fun with it
                //MessageBox.Show("Could not send your post to Twitter, will try again in a few :-)" + Environment.NewLine + e.Error.Message);
            }
        }

        #endregion
    }
}
