﻿using System;
using System.IO;
using System.Linq;
using System.Net;
using System.ServiceModel.Activation;
using System.ServiceModel.Web;
using System.Text;
using System.Web;
using System.Xml.Linq;

namespace SilverTwitMSDNWS
{
    [AspNetCompatibilityRequirements(RequirementsMode = AspNetCompatibilityRequirementsMode.Allowed)]
    public class TwitterRESTService : ITwitterRESTService
    {
        #region Constants

        private const string TWEET_SOURCE = "SilverTwit";

        #endregion

        public TwitterRESTService()
        {
            ServicePointManager.Expect100Continue = false;
        }

        #region Public Web Methods

        public XElement FindTweets(string username, string tweetType, string password)
        {
            OutgoingWebResponseContext outResponse = WebOperationContext.Current.OutgoingResponse;

            if (string.IsNullOrEmpty(username) || string.IsNullOrEmpty(password) || string.IsNullOrEmpty(tweetType))
            {
                WebOperationContext ctx = WebOperationContext.Current;
                ctx.OutgoingResponse.StatusCode = HttpStatusCode.BadRequest;
                return null;
            }

            string uriString = string.Empty;
            switch (tweetType)
            {
                case "all":
                    uriString = string.Format(@"http://twitter.com/statuses/friends_timeline/{0}.xml?count=50", username);
                    break;
                case "dm":
                    uriString = string.Format(@"http://twitter.com/direct_messages.xml");
                    break;
                case "replies":
                    uriString = string.Format(@"http://twitter.com/statuses/replies.xml");
                    break;
                default:
                    outResponse.StatusCode = HttpStatusCode.BadRequest;
                    return null;
                    break;
            }
            outResponse.StatusCode = HttpStatusCode.OK;
            return GetRequest(uriString, username, password, outResponse);
        }

        public XElement FindPublicTweets()
        {
            OutgoingWebResponseContext outResponse = WebOperationContext.Current.OutgoingResponse;
            return GetRequest(@"http://twitter.com/statuses/public_timeline.xml", outResponse);
        }

        public void AddTweet(string username, string password, TweetStatus newTweetStatus)
        {
            //System.Net.ServicePointManager.Expect100Continue = false;
            string uri = @"http://twitter.com/statuses/update.xml";
            IncomingWebRequestContext inRequest = WebOperationContext.Current.IncomingRequest;
            OutgoingWebResponseContext outResponse = WebOperationContext.Current.OutgoingResponse;
            //WebOperationContext ctx = WebOperationContext.Current;
            WebRequest webRequest = HttpWebRequest.Create(uri);

            if (newTweetStatus.Text == null)
            {
                outResponse.StatusCode = HttpStatusCode.BadRequest;
                return;
            }

            string message = newTweetStatus.Text;
            message = message.Trim();
            if (message.Length > 140)
            {
                outResponse.StatusCode = HttpStatusCode.BadRequest;
                return;
            }

            // The TWEET_SOURCE constant below is a class level constant, set to "SilverTwit".
            // It defines the name of the source program talking to Twitter.
            string parameters = string.Format("status={0}&source={1}", HttpUtility.HtmlEncode(message), TWEET_SOURCE);

            if (string.IsNullOrEmpty(username) || string.IsNullOrEmpty(password))
            {
                outResponse.StatusCode = HttpStatusCode.BadRequest;
                return;
            }

            // The username and password variables below are set as fields of this class.
            webRequest.Credentials = new NetworkCredential(username, password);
            webRequest.ContentType = "application/x-www-form-urlencoded";
            webRequest.Method = "POST";
            ASCIIEncoding encoding = new ASCIIEncoding();
            byte[] bytes = encoding.GetBytes(parameters);

            Stream os = null;
            try
            {
                webRequest.ContentLength = bytes.Length;
                os = webRequest.GetRequestStream();
                os.Write(bytes, 0, bytes.Length);
            }
            catch (WebException ex)
            {
                // Not needed really, since 500 is the default status code for an exception
                outResponse.StatusCode = HttpStatusCode.InternalServerError;
                return;
            }
            finally
            {
                if (os != null)
                    os.Close();
            }

            try
            {
                WebResponse webResponse = webRequest.GetResponse();
                //if (webResponse == null)
                //{
                //    return;
                //}

                //StreamReader sr = new StreamReader(webResponse.GetResponseStream());
                //returnData = sr.ReadToEnd().Trim();
                //sr.Close();
            }
            catch (WebException ex)
            {
                outResponse.StatusCode = HttpStatusCode.InternalServerError;
                return;
            }

            // Set the outgoing status and response headers
            //outResponse.StatusCode = HttpStatusCode.Created;
            UriTemplate template = new UriTemplate("Tweet/{username}/?pw={password}");
            Uri newUri = template.BindByPosition(inRequest.UriTemplateMatch.BaseUri, username, password);
            outResponse.SetStatusAsCreated(newUri);
            return;
        }

        #endregion

        #region Private Helper Methods

        private XElement GetRequest(string uri, OutgoingWebResponseContext outResponse)
        {
            return GetRequest(uri, string.Empty, string.Empty, outResponse);
        }

        private XElement GetRequest(string uri, string username, string password, OutgoingWebResponseContext outResponse)
        {
            try
            {
                SetNoCache(outResponse);

                WebClient wc = new WebClient();
                if (username.Length > 0 && password.Length > 0)
                    wc.Credentials = new NetworkCredential(username, password);
                string data = wc.DownloadString(uri);
                XDocument xdoc = XDocument.Parse(data);

                XElement xmlElement;
                if (xdoc.Descendants("direct-messages").Any())
                    xmlElement = xdoc.Element("direct-messages");
                else
                    xmlElement = xdoc.Element("statuses");

                outResponse.StatusCode = HttpStatusCode.OK;
                return xmlElement;
            }
            catch (Exception ex)
            {
                //TODO: Do something better :) ... log the error too
                // Eat the error for now
                Console.WriteLine(ex.Message);
                return null;
            }
        }

        private void SetNoCache(OutgoingWebResponseContext outResponse)
        {
            //Tell client not to cache
            outResponse.Headers.Add("Pragma", "no-cache");
            outResponse.Headers.Add("Cache-Control", "must-revalidate");
            outResponse.Headers.Add("Cache-Control", "no-cache");
            outResponse.Headers.Add("Cache-Control", "no-store");
            outResponse.Headers.Add("Expires", "0");
        }
        
        #endregion
    }
}