﻿using System;
using System.IO;
using System.Net;
using System.ServiceModel;
using System.ServiceModel.Activation;
using System.ServiceModel.Web;
using System.Text;
using System.Web;

namespace SilverTwitWS
{
    [AspNetCompatibilityRequirements(RequirementsMode = AspNetCompatibilityRequirementsMode.Allowed)]
    public class TwitterService : ITwitterService
    {
        private const string tweetSource = "SilverTwit";

        public string FindTwitterTweets()
        {
            WebClient wc = new WebClient();
            string uri = string.Format(@"http://twitter.com/statuses/public_timeline.xml");
            string data = wc.DownloadString(uri);
            return data;
        }

        public string FindTweets(string username, string password)
        {
            return GetRequest(string.Format(@"http://twitter.com/statuses/friends_timeline/{0}.xml", username), username, password);
        }

        public string FindPublicTweets()
        {
            return GetRequest(@"http://twitter.com/statuses/public_timeline.xml");
        }

        public string FindArchivedTweets(string username, string password)
        {
            return GetRequest(string.Format(@"http://twitter.com/statuses/user_timeline/{0}.xml", username), username, password);
        }

        public string FindReplies(string username, string password)
        {
            return GetRequest(@"http://twitter.com/statuses/replies.xml", username, password);
        }

        public string FindDirectMessages(string username, string password)
        {
            return GetRequest(@"http://twitter.com/direct_messages.xml", username, password);
        }

        public string FindFavorites(string username, string password)
        {
            return GetRequest(@"http://twitter.com/favourings.xml", username, password);
        }

        public string PostStatusUpdate(string message, string username, string password)
        {
            message = message.Trim();
            if (message.Length > 140)
            {
                message = message.Substring(0, 140);
            }

            string parameters = string.Format("status={0}&source={1}", HttpUtility.HtmlEncode(message), tweetSource);

            return PostRequest(@"http://twitter.com/statuses/update.xml", username, password, parameters);
        }

        public string PostCreateTinyUrl(string longUrl)
        {
            return PostRequest(string.Format(@"http://tinyurl.com/api-create.php?url={0}/", longUrl));
        }

        public bool AuthenticateUser(string username, string password)
        {
            try
            {
                string temp = GetRequest(@"http://twitter.com/account/verify_credentials.xml", username, password);
                return true;
            }
            catch (Exception ex)
            {
                return false;
            }
        }

        #region Private Helper Methods

        private string GetRequest(string uri)
        {
            return GetRequest(uri, string.Empty, string.Empty);
        }

        private string GetRequest(string uri, string username, string password)
        {
            OutgoingWebResponseContext responseContext = WebOperationContext.Current.OutgoingResponse;
            responseContext.Headers.Add("Cache-Control", "no-cache");
            responseContext.Headers.Add("Pragma", "no-cache");
            responseContext.Headers.Add("Expires", GetFormattedCacheTime(10)); //TODO: Write the method to set this in seconds

            bool useWebClient = false;
            if (useWebClient)
            {
                WebClient wc = new WebClient();
                if (!string.IsNullOrEmpty(username) && !string.IsNullOrEmpty(password))
                {
                    wc.Credentials = new NetworkCredential(username, password);
                }
                string data = wc.DownloadString(uri);
                responseContext.StatusCode = HttpStatusCode.OK;
                return data;
            }
            else
            {
                WebRequest request = HttpWebRequest.Create(uri);

                if (!string.IsNullOrEmpty(username) && !string.IsNullOrEmpty(password))
                {
                    request.Credentials = new NetworkCredential(username, password);
                }

                WebResponse res = request.GetResponse();
                StreamReader stream = new StreamReader(res.GetResponseStream());
                string data = stream.ReadToEnd();
                stream.Close();
                responseContext.StatusCode = HttpStatusCode.OK;
                return data;
            }
        }

        private string GetFormattedCacheTime(int i)
        {
            string value = DateTime.Now.Add(new TimeSpan(0, 0, 0, i)).ToLongDateString();
            return value;
        }

        private string PostRequest(string uri)
        {
            return PostRequest(uri, string.Empty, string.Empty, string.Empty);
        }

        private string PostRequest(string uri, string username, string password, string parameters)
        {
            string returnData;
            WebRequest webRequest = HttpWebRequest.Create(uri);

            if (!string.IsNullOrEmpty(username) && !string.IsNullOrEmpty(password))
            {
                webRequest.Credentials = new NetworkCredential(username, password);
            }

            webRequest.ContentType = "application/x-www-form-urlencoded";
            webRequest.Method = "POST";
            ASCIIEncoding encoding = new ASCIIEncoding();
            byte[] bytes = encoding.GetBytes(parameters);

            Stream os = null;
            try
            {
                webRequest.ContentLength = bytes.Length;
                os = webRequest.GetRequestStream();
                os.Write(bytes, 0, bytes.Length);
            }
            catch (WebException ex)
            {
                throw ex;
            }
            finally
            {
                if (os != null)
                {
                    os.Close();
                }
            }

            try
            {
                WebResponse webResponse = webRequest.GetResponse();
                if (webResponse == null)
                {
                    return null;
                }

                StreamReader sr = new StreamReader(webResponse.GetResponseStream());
                returnData = sr.ReadToEnd().Trim();
                sr.Close();
            }
            catch (WebException ex)
            {
                throw ex;
            }

            return returnData;
        }

        #endregion
    }
}

